import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

export class CriApi {
	token;
	constructor(token: string) {
		this.token = token;
	}

	async groups_of(login: string) {
		const response = await fetch(`https://cri.epita.fr/api/v2/users/${login}/`, {
			headers: {
				accept: "application/json",
				authorization: "Basic " + this.token,
			},
		});
		const body = await response.json();
		const group_parser = z.object({ slug: z.string() });
		const parser = z.object({
			primary_group: group_parser,
			groups_history: z.array(z.object({
				group: group_parser,
				graduation_year: z.number().or(z.null()),
			})),
			current_groups: z.array(group_parser),
		});
		const parsed = parser.parse(body);
		const result = new Set<string>();
		result.add(parsed.primary_group.slug);
		for (const group of parsed.current_groups) result.add(group.slug);
		for (const { group } of parsed.groups_history) result.add(group.slug);
		return Array.from(result.values());
	}

	async user_exists(login: string) {
		const response = await fetch(`https://cri.epita.fr/api/v2/users/${login}/`, {
			headers: {
				accept: "application/json",
				authorization: "Basic " + this.token,
			},
		});
		return response.status !== 404;
	}
}
