import * as path from "https://deno.land/std@0.213.0/path/mod.ts";
import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

export function root_path() {
	const this_path = new URL(import.meta.url).pathname;
	return path.resolve(this_path, "..", "..");
}

export function channel<T>() {
	const inner = {
		items: [] as T[],
		awaiters: [] as ((item: T) => void)[],
	};

	function send(item: T) {
		const [awaiter] = inner.awaiters.splice(0, 1);
		if (awaiter !== undefined) awaiter(item);
		else inner.items.push(item);
	}

	function receive() {
		const [item] = inner.items.splice(0, 1);
		if (item !== undefined) return resolves_to(item);
		else return new Promise<T>((resolver) => inner.awaiters.push(resolver));
	}

	return { send, receive };
}

const resolves_to = <T>(item: T) => new Promise<T>((r) => r(item));

export function split_promise<T>() {
	let resolver: null | ((item: T) => void) = null;
	const promise = new Promise<T>((r) => resolver = r);
	resolver = resolver as unknown as (item: T) => void;
	return { promise, resolver };
}

export function log_from(url: string, ...args: unknown[]) {
	const date = new Date().toLocaleString("fr-FR");
	const file = path.basename(new URL(url).pathname);
	console.log(`[${date}][epitls][${file}]`, ...args);
}

export async function read_secrets(path: string) {
	const content = await Deno.readTextFile(path);
	return z.object({
		discord_bot_token: z.string(),
		cri_token: z.string(),
	}).parse(JSON.parse(content));
}

export type SimpleResult = true | string;

export async function wait(ms: number) {
	const { promise, resolver } = split_promise<void>();
	setTimeout(resolver, ms);
	await promise;
}
